#include <unistd.h>
#include <stdlib.h>
#include <qapplication.h>
#include <qdialog.h>
#include <qlistview.h>
#include <qpushbutton.h>
#include <qcombobox.h>
#include <qlineedit.h>
#include <qspinbox.h>
#include <qlayout.h>
#include <qhbox.h>
#include <qvbox.h>
#include <qaction.h>
#include <qcheckbox.h>
#include <qprogressbar.h>
#include <qpe/qpetoolbar.h>
#include <qmessagebox.h>
#include <qpe/qpemenubar.h>
#include <qpe/config.h>
#include <qpe/resource.h>


#include "categories.h"
#include "accountlist.h"
#include "accountlistwidget.h"
#include "accountview.h"
#include "lineinput.h"
#include "changepassword.h"
#include "createcategory.h"
#include "settingsui.h"
#include "importaccountsui.h"

#include "resettimer.xpm"
#include "lock.xpm"
#include "changeview.xpm"

/* This object is used to represent category roots
 * on the account view tree.
 * 
 * NCC: add support for case-insensitive sort
 */
class AccountCategoryItem : public QListViewItem
{
	public:
	AccountCategoryItem(Config *config, QListView * parent,QString name) :
						QListViewItem( parent,name ), mConfig(config) { 
						
		QListViewItem::setOpen(mConfig->readBoolEntry("category."+text(0),true));					
	}

	virtual QString key( int column, bool ascending ) const
	{
		return QListViewItem::key( column, ascending ).lower();
	}

	virtual void setOpen(bool open){
		mConfig->writeEntry("category."+text(0),open);	
		QListViewItem::setOpen(open);
	}
	Config *mConfig;
};

/* This object is used both on the tree and list view
 * to represent either accounts or accounts and categories.
 * 
 * NCC: add support for case-insensitive sort
 */
class AccountListItem : public QListViewItem
{
	public:
	AccountListItem( QListView * parent, QString label,QString category ) :
						QListViewItem( parent, label, category ) { }
	
	AccountListItem( QListViewItem *root, QString label,QString category ) :
						QListViewItem( root, label, category ) { }

	virtual QString key( int column, bool ascending ) const
	{
		return QListViewItem::key( column, ascending ).lower();
	}
};


/* Constructor for the accountlist.
 *
 * The account list is the main screen when you start
 * keyring. This is what the user first sees.
 *
 */ 
AccountList::AccountList( PasswordStore *ps,QWidget* parent,  const char* name, WFlags fl )
	:QMainWindow( parent, name, fl ),mPStore(ps)
{
	/* open up our config data */
	mConfig = new Config("keyring");
	mConfig->setGroup("Main");
		
		
	setCaption(tr("Keyring: Account List"));
	
	setToolBarsMovable( FALSE );

	/* create all the nice gui bits */
	mToolBar = new QPEToolBar( this );
	mToolBar->setHorizontalStretchable( TRUE );

	mMenuBar = new QPEMenuBar( mToolBar );
	mMenuBar->setMargin( 0 );
   
	QPopupMenu *accounts= new QPopupMenu(this);
	QPopupMenu *tools= new QPopupMenu(this);

	mMenuBar->insertItem(tr("Accounts"),accounts);
	
	mMenuBar->insertItem(tr("Tools"),tools);
	
  
	mAccountActions = new QActionGroup(this);
	
	/* -- Accounts Menu -- */
	QAction *action = new QAction(tr("New"), Resource::loadPixmap("new"),
			QString::null,0,mAccountActions,0);
	action->addTo(accounts); 
	action->addTo(mToolBar); 
	connect(action,SIGNAL(activated()),this,SLOT(newAccount()));
	
	action = new QAction(tr("Lock"), QPixmap((const char **) lock_xpm),
			QString::null,0,mAccountActions,0);
	action->addTo(accounts); 
	action->addTo(mToolBar); 
	connect(action,SIGNAL(activated()),this,SLOT(lock()));

	action = new QAction(tr("Reset Timer"), QPixmap((const char **) resettimer_xpm),
			QString::null,0,mAccountActions,0);
	action->addTo(accounts); 
	action->addTo(mToolBar); 
	connect(action,SIGNAL(activated()),this,SLOT(resetTimer()));

	action = new QAction(tr("Change View"), QPixmap((const char **) changeview_xpm),
			QString::null,0,mAccountActions,0);
	action->addTo(accounts); 
	action->addTo(mToolBar); 
	connect(action,SIGNAL(activated()),this,SLOT(changeView()));
	

	/* -- Tools Menu -- */
	action = new QAction(tr("Settings"),tr("Settings"),0,mAccountActions);
	action->addTo(tools); 
	connect(action,SIGNAL(activated()),this,SLOT(settings()));
	
	action = new QAction(tr("Change Password"),tr("Change Password"),0,mAccountActions);
	action->addTo(tools); 
	connect(action,SIGNAL(activated()),this,SLOT(changePassword()));
	
	mCategoryEdit = new QAction(tr("Edit Category"),tr("Edit Category"),0,mAccountActions);
	mCategoryEdit->addTo(tools); 
	mCategoryEdit->setEnabled(false);
	connect(mCategoryEdit,SIGNAL(activated()),this,SLOT(editCategory()));

	tools->insertSeparator();
	
	action = new QAction(tr("Export Accounts"),tr("Export Accounts"),0,mAccountActions);
	action->addTo(tools); 
	connect(action,SIGNAL(activated()),this,SLOT(exportAccounts()));
	
	action = new QAction(tr("Import Accounts"),tr("Import Accounts"),0,mAccountActions);
	action->addTo(tools); 
	connect(action,SIGNAL(activated()),this,SLOT(importAccounts()));


	
	/* -- Rest of the widgets --*/
	AccountListWidget *aw = new AccountListWidget(this);

	wPassword = aw->wPassword;
	wAccounts = aw->wAccounts;
	wLCD = aw->wLCD;
	
	wLCD->display(0);
	
	setCentralWidget(aw);
	
	mAccountActions->setEnabled(false);
	wAccounts->setEnabled(false);

	wAccounts->setColumnWidthMode(0,QListView::Maximum);
	wAccounts->setColumnWidth(0,185);
	wAccounts->setColumnText(0,tr("Name of Account"));
		
	/* set up our timer 
	 *
	 * NCC tells me there is signal that can
	 * also be used to determine when to time
	 * out the device, right after it has
	 * been powered back on.
	 */
	mUpdateTimer = new QTimer(this);
	connect(mUpdateTimer,SIGNAL(timeout()),this,SLOT(updateCountDown()));

	connect(wPassword,SIGNAL(textChanged(const QString &)),this,SLOT(passwordEntered(const QString &)));
	connect(wAccounts,SIGNAL(clicked(QListViewItem *,const QPoint &, int)),
					this,SLOT(selectAccount(QListViewItem *,const QPoint &, int)));
	connect(wAccounts,SIGNAL(returnPressed(QListViewItem *)),this,SLOT(selectAccount(QListViewItem *)));
	connect(wAccounts,SIGNAL(selectionChanged(QListViewItem *)),this,SLOT(selectionChanged(QListViewItem *)));
	
	/* Read our settings */
	applySettings();

	/* Figure out weither or not
	 * we should show the account
	 * list at startup */
	showAccountList();

	
}

/* Show the list/tree of accounts 
 *
 * This function shows the list or tree of
 * accounts. It tries to be relatively smart 
 * about setting up the QListView
 */
void AccountList::showAccountList(){
		
		wAccounts->clear();

		if(mHideAccounts){
			if(wAccounts->isEnabled()==false)
				return;
		}

		
		/* Make sure we can open the password db */
		if(!mPStore){
			Error("Failed to open password database");
			return;
		}
	
		/* Are we showing the list ? */
		if(mAccountList){
			wAccounts->setRootIsDecorated(false);

				
			/* Set up our columns */
			if(wAccounts->columns()==1){
				wAccounts->addColumn(tr("Category"));	
			
				wAccounts->setColumnWidth(0,mConfig->readNumEntry("list.column0.width",155));
				wAccounts->setColumnWidth(1,mConfig->readNumEntry("list.column1.width",55));
				wAccounts->setColumnWidthMode(0,QListView::Maximum);
				wAccounts->setColumnWidthMode(1,QListView::Maximum);
			}

			/* show all the items */
			QStringList::Iterator i;
			QStringList items = mPStore->entries();
			for(i=items.begin();i!=items.end();i++){
				//Get the category for the item
				QString category = mPStore->getCategory((*i));
				wAccounts->insertItem(new AccountListItem(wAccounts,(*i),category));
			}
		
		} else {
			//Ok, we are going to show a tree
			// set up our columns
			if(wAccounts->columns()>1){
				wAccounts->removeColumn(1);
				wAccounts->setColumnWidthMode(0,QListView::Maximum);
			}
			wAccounts->setColumnWidth(0,mConfig->readNumEntry("tree.column0.width",185));
			
			wAccounts->setRootIsDecorated(true);
			QStringList list = mPStore->categoryList();
			QStringList items = mPStore->entries();
			//Make it so that items without categories show
			//up!
			if(list.contains("Default")==0){
				list.prepend("Default");
			}

			/* This is really ineffiecent. We get a list of all our categories
			 *  which we use to iterate through all of our accounts and decide
			 *  which account goes in which group. ieee!!!!
			 */
			for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it ) {
				QString pname = mPStore->getCategoryPixmap((*it));
				QListViewItem *root = new AccountCategoryItem(mConfig,wAccounts,(*it));
				if(pname != "none"){
					QPixmap *pixmap = getCategoryPixmapByName(pname);
					if(pixmap){
						root->setPixmap(0,*pixmap);
					}
				}  
				int count=0;	
				//Go through our accounts and figure out which ones belong 
				//in this category.
				QStringList::Iterator i;
				for(i=items.begin();i!=items.end();i++){
					QString cat = mPStore->getCategory((*i));
					if(qstrcmp(cat,(*it))==0){
						new AccountListItem(root,(*i),cat);
						count++;
					}
				}
				if(count>0){
					wAccounts->insertItem(root);
				} else {
					delete root;
				}
			}
		}
}

/* reset the timer
 */
void AccountList::resetTimer(){
	mSeconds=mTimerStart;	
	mStartTime = time(NULL);
	wLCD->display(mSeconds);	
}

/* switch view between tree and list 
 */
void AccountList::changeView(){
	if(mAccountList){
		mConfig->writeEntry("list.column0.width",wAccounts->columnWidth(0));
		mConfig->writeEntry("list.column1.width",wAccounts->columnWidth(1));
	} else {
		mConfig->writeEntry("tree.column0.width",wAccounts->columnWidth(0));
	}
	mAccountList=!mAccountList;
	showAccountList();
	mConfig->writeEntry("accountlist",mAccountList);
}

/*  
 *  Destroys the object and frees any allocated resources
 */
AccountList::~AccountList()
{
	//Save column width preferences
	if(mAccountList){
		mConfig->writeEntry("list.column0.width",wAccounts->columnWidth(0));
		mConfig->writeEntry("list.column1.width",wAccounts->columnWidth(1));
	} else {
		mConfig->writeEntry("tree.column0.width",wAccounts->columnWidth(0));
	}
	// no need to delete child widgets, Qt does it all for us
	qDebug("Deleting AccountList");
	delete mConfig;
}

/*  lock keyring
 */
void AccountList::lock()
{
	qDebug("Locking database");
		
	mPStore->lock();
	wPassword->setText("");

	
	mAccountActions->setEnabled(false);
	mCategoryEdit->setEnabled(false);
	wAccounts->setEnabled(false);
	
	showAccountList();

	if(mTimerEnabled)
		mUpdateTimer->stop();
}


/* create an account 
 */
void AccountList::newAccount()
{
	if(mTimerEnabled)
		mUpdateTimer->stop();
	
	time_t viewtime=time(NULL);

	AccountView *qd = new AccountView(*mPStore,mConfig,0,0,true);
	if(!qd){
		Error("Failed to allocate account view window");
	}
	qd->exec();
	
	//Let see if we were supposed to keep track
	//of the timer while the account was being viewed
	//if so and it has expired, lock keyring.
	if(mTimerViewingAC && mTimerEnabled){
			//Timer has expired
			if((time(NULL)-mStartTime) > mTimerStart){
					lock();
			} else {
					//Timer needs to be updated
					time_t diff=time(NULL)-viewtime;
					mSeconds=mSeconds-diff;
					mStartTime = time(NULL)-(mTimerStart-mSeconds);
					mUpdateTimer->start(500,false);
					wLCD->display(mSeconds);	
			}
	} else {
			if(mTimerEnabled){
					mUpdateTimer->start(500,false);
					mStartTime = time(NULL)-(mTimerStart-mSeconds);
			}
	}

	delete qd;
	showAccountList();
	
}


/* the password was just entered
 */
void AccountList::passwordEntered(const QString &str)
{
	if(!mPStore){
		Error("Failed to open password database");
		return;
	}
	
	if(mPStore->unlock(str)){
		mAccountActions->setEnabled(true);
		mCategoryEdit->setEnabled(false);
		wAccounts->setEnabled(true);
		
		showAccountList();
		
		if(mTimerEnabled)
			mUpdateTimer->start(500,false);
		mSeconds=mTimerStart;
		mStartTime = time(NULL);
		updateCountDown();
	
		//Let's be a bastard and bitch if the
		//Last file where accounts were exported still exists!
		QString exportfile = mConfig->readEntry("exportfile");
		if(exportfile != QString::null){
			if(access(exportfile.latin1(),F_OK)==0){
				QString msg;
				msg.sprintf("The file %s\n from the last time\n account information was\n exported still exists!!.\n Do you want to delete it?",exportfile.latin1());
				if(QMessageBox::Yes==QMessageBox::critical(this,tr("Warning!!"),msg,QMessageBox::Yes,QMessageBox::No)){
					unlink(exportfile.latin1());
				}
			}
		}
		
	} else {
		wLCD->display(0);
		if(mTimerEnabled)
			mUpdateTimer->stop();
			
		mAccountActions->setEnabled(false);
		wAccounts->setEnabled(false);
		showAccountList();
	}
}


void AccountList::updateCountDown(){
	wLCD->display(mSeconds);	
	if(mSeconds==0 || 
			(((time(NULL)-mStartTime) > mTimerStart) && mTimerPowerOff)){
		lock();
		return;
	}
	mSeconds=mTimerStart-(time(NULL)-mStartTime);
}

void AccountList::selectionChanged(QListViewItem *item){
	if(item){	
		if(!mAccountList){ 
			if(item->parent()==NULL){
				mCategoryEdit->setEnabled(true);	
				return;
			} else {
				mCategoryEdit->setEnabled(false);
			}
		} else {
			mCategoryEdit->setEnabled(true);
		}
	}

}

void AccountList::editCategory(){
	QString category;
	if(wAccounts->selectedItem()){
		if(!mAccountList){
			category = wAccounts->selectedItem()->text(0);
		} else {
			category = wAccounts->selectedItem()->text(1);
		}
		CreateCategory::EditCategory(this,mPStore,category);
		showAccountList();
	}
}

void AccountList::selectAccount(QListViewItem *item)
{
	selectAccount(item,QPoint(0,0),0);
}


/* 
 * public slot
 */
void AccountList::selectAccount(QListViewItem *item,const QPoint &, int c)
{
	
	if(item){	
		if(!mAccountList){ 
			if(item->parent()==NULL){
				mCategoryEdit->setEnabled(true);	
				return;
			} else {
				mCategoryEdit->setEnabled(false);
			}
		} else {
			mCategoryEdit->setEnabled(true);

			if(c==1){
				CreateCategory::EditCategory(this,mPStore,item->text(1));
				showAccountList();
				return;
			}
			
		}

		
		
			
		if(mTimerEnabled)
			mUpdateTimer->stop();
		timer_t viewtime=time(NULL);
		AccountView *qd = new AccountView(*mPStore,mConfig,0,0,true);
		
	
		qd->loadAccount(item->text(0));
	
		qd->exec();

		//Let see if we were supposed to keep track
		//of the timer while the account was being viewed
		//if so and it has expired, lock keyring.
		if(mTimerViewingAC && mTimerEnabled){
			//Timer has expired
			if((time(NULL)-mStartTime) > mTimerStart){
				lock();
			} else {
				//Timer needs to be updated
				time_t diff=time(NULL)-viewtime;
				mSeconds=mSeconds-diff;
				mStartTime = time(NULL)-(mTimerStart-mSeconds);
				mUpdateTimer->start(500,false);
				wLCD->display(mSeconds);	
			}
		} else {
			if(mTimerEnabled){
				mUpdateTimer->start(500,false);
				mStartTime = time(NULL)-(mTimerStart-mSeconds);
			}
		}
		delete qd;	
		showAccountList();
	}
}

void AccountList::applySettings(){
	mTimerStart=mConfig->readNumEntry("timervalue",60);
	mTimerEnabled=mConfig->readBoolEntry("timerenabled",true);
	mTimerPowerOff=mConfig->readBoolEntry("timerpoweroff",true);
	mHideAccounts=mConfig->readBoolEntry("hideaccounts",false);
	mAccountList=mConfig->readBoolEntry("accountlist",false);
	mTimerViewingAC=mConfig->readBoolEntry("timerviewingaccount",true);
	
	if(mConfig->readBoolEntry("hidepassword",true))
		wPassword->setEchoMode(QLineEdit::Password);
	else
		wPassword->setEchoMode(QLineEdit::Normal);

	if(mAccountActions->isEnabled()){
			if(!mTimerEnabled)
					mUpdateTimer->stop();
			else
					mUpdateTimer->start(500,false);
	}
}

void AccountList::settings(){
	if(mTimerEnabled)
			mUpdateTimer->stop();

	SettingsUI settings(this,"settings",TRUE);

	settings.showMaximized();

	settings.wTimerValue->setValue(mTimerStart);
	settings.wEnableTimer->setChecked(mTimerEnabled);
	settings.wTimerPowerOff->setChecked(mTimerPowerOff);

	settings.wHidePassword->setChecked(wPassword->echoMode() == QLineEdit::Password);
	settings.wHideAccounts->setChecked(mHideAccounts);
	settings.wAccountList->setChecked(mAccountList);

	settings.wAVEnableTimer->setChecked(mConfig->readBoolEntry("av.timerenabled",true));
	settings.wAVTimerValue->setValue(mConfig->readNumEntry("av.timervalue",60));

	settings.wTimerViewingAC->setChecked(mTimerViewingAC);
	
	
	if(settings.exec()==QDialog::Accepted){
		mConfig->writeEntry("timervalue",settings.wTimerValue->value());
		mConfig->writeEntry("timerenabled",settings.wEnableTimer->isChecked());
		mConfig->writeEntry("timerpoweroff",settings.wTimerPowerOff->isChecked());
		mConfig->writeEntry("hidepassword",settings.wHidePassword->isChecked());
		mConfig->writeEntry("hideaccounts",settings.wHideAccounts->isChecked());
		mConfig->writeEntry("accountlist",settings.wAccountList->isChecked());
		mConfig->writeEntry("av.timerenabled",settings.wAVEnableTimer->isChecked());
		mConfig->writeEntry("av.timervalue",settings.wAVTimerValue->value());
		mConfig->writeEntry("timerviewingaccount",settings.wTimerViewingAC->isChecked());
		applySettings();
	}
		
	if(mTimerEnabled){
			mUpdateTimer->start(500,false);
			mStartTime = time(NULL)-(mTimerStart-mSeconds);
	}

	showAccountList();

}

void AccountList::changePassword(){
	ChangePassword cp(this,"changepassword",TRUE);

	cp.showMaximized();
	
	if(mTimerEnabled)
			mUpdateTimer->stop();
	
	if(cp.exec()==QDialog::Accepted){
		int ret=QMessageBox::warning(this,tr("Change Password?"),tr("Are you sure you want\nto change the password?"),QMessageBox::Yes,QMessageBox::No);
		if(ret==QMessageBox::Yes){
			QDialog db(this,"Progress",true);
			QProgressBar pb(&db);
		
			db.show();
	
			QString newpassword = cp.wPassword1->text();
		
			QStringList::Iterator i;
			QStringList items = mPStore->entries();

			pb.setTotalSteps(items.count());
			int j=0;	
			for(i=items.begin();i!=items.end();i++){
				qDebug("Changing password");
				qDebug((*i));
				if(!mPStore->changePassword((*i),newpassword)){
					QMessageBox::critical(this,tr("Error"),tr("An error has been\nencountered while\nchanging your password\n"),QMessageBox::Ok,QMessageBox::NoButton);
					break;
				}
				//Give the user some sense that something is happening
				usleep(100);
				pb.setProgress(j++);
			}
			mPStore->initPassword(newpassword);
			qDebug("Password change complete");
		}
	} 

	lock();
}

QString text2xmlEscape(QString in){
	in.replace(QRegExp("&"),"&amp;");
	in.replace(QRegExp(">"),"&gt;");
	in.replace(QRegExp("<"),"&lt;");
	return in;
}

void AccountList::exportAccounts(){

			if(QMessageBox::No==QMessageBox::warning(this,tr("Warning"),
				tr("You are about to export\nyour account information\nto a unprotected file!\nDo you want to continue?"),QMessageBox::Yes,QMessageBox::No)){
				return;
			}
		
			LineInput li(this,tr("Export File"));
			if(li.exec() == QDialog::Accepted){

					
				FILE *out = fopen(li.text().latin1(),"wt");
				if(out){		
					//Ok lets make sure this user knows
					//the password to export the data
					lock();
					mConfig->removeEntry("exportfile");
						
					LineInput pwd(this,tr("Password"));
					pwd.lineEdit()->setEchoMode(QLineEdit::Password);
					if(pwd.exec() == QDialog::Accepted){
						if(!mPStore->unlock(pwd.text())){
							QMessageBox::critical(this,tr("Error"),tr("Incorrect Password"),QMessageBox::Ok,QMessageBox::NoButton);
							return;
						}
						passwordEntered(pwd.text());
					} else {
						return;	
					}
					
					
					fprintf(out,"<keyring>\n");
					QStringList::Iterator i;
					QStringList items = mPStore->entries();
					for(i=items.begin();i!=items.end();i++){
						//Get the category for the item
						QString category = mPStore->getCategory((*i));
						PasswordData *data = mPStore->load(*i);
						if(data){
								fprintf(out,"\t<account>\n");
								fprintf(out,"\t\t<name>%s</name>\n",text2xmlEscape((*i)).latin1());
								fprintf(out,"\t\t<category>%s</category>\n",text2xmlEscape(category).latin1());
								fprintf(out,"\t\t<user>%s</user>\n",text2xmlEscape(data->mUser).latin1());
								fprintf(out,"\t\t<password>%s</password>\n",text2xmlEscape(data->mPassword).latin1());
								fprintf(out,"\t\t<note>%s</note>\n",text2xmlEscape(data->mNote).latin1());
								fprintf(out,"\t</account>\n");
						}
						delete data;
					}
					fprintf(out,"</keyring>\n");
					fclose(out);
					QMessageBox::information(this,tr("Information"),tr("Accounts Exported!\nPlease delete this file when\nyou're done with it."),QMessageBox::Ok);
					mConfig->writeEntry("exportfile",li.text());
				} else {
					QMessageBox::critical(this,tr("Error"),tr("Unable to open file"),QMessageBox::Ok,QMessageBox::NoButton);
				}
			}
}

QString xml2textEscape(QString in){
	in.replace(QRegExp("&amp;"),"&");
	in.replace(QRegExp("&gt;"),">");
	in.replace(QRegExp("&lt;"),"<");
	return in;
}

void AccountList::importAccounts(){
			
			bool overwrite=false;
			ImportAccountsUI iau(this,"import",true);
			if(iau.exec() == QDialog::Accepted){

				overwrite=iau.wOverwrite->isChecked();
					
				FILE *in = fopen(iau.wFilename->text().latin1(),"rt");
				if(in){		
					//Ok lets make sure this user knows
					//the password to export the data
					lock();
						
					LineInput pwd(this,tr("Password"));
					pwd.lineEdit()->setEchoMode(QLineEdit::Password);
					if(pwd.exec() == QDialog::Accepted){
						if(!mPStore->unlock(pwd.text())){
							QMessageBox::critical(this,tr("Error"),tr("Incorrect Password"),QMessageBox::Ok,QMessageBox::NoButton);
							return;
						}
						passwordEntered(pwd.text());
					} else {
						return;	
					}

					char tag[50];
					char etag[50];
					int  ti=0;
					char cdata[9000];
					int di=0;
					int newaccts=0,renaccts=0,overacct=0;
					PasswordData *pData=NULL;
					char *cat=NULL;

					//Lets parse this xml file
					while(!feof(in)){
						char c=fgetc(in);
						//Look for the start of a tag
						if(c == '<'){
							ti=0;

							//Lets see if we can find the end
							//of this tag
							if(!feof(in) && (c=fgetc(in))!='>'){

								//Is this an end-tag?
								if(c =='/'){
									while(!feof(in) && (c=fgetc(in))!='>'){
										if(ti<50){
											etag[ti++]=c;
										}
									}
									etag[ti]='\0';

									
									//Found the end of an account
									if(strcasecmp(etag,"account")==0){
										if(pData){
											//Don't Overwrite any existing
											//accounts
											if(mPStore->accountExists(pData->mName)){
												if(!overwrite){
													pData->mName=pData->mName+".new";
													renaccts++;
												} else {
													overacct++;
												}
											} else newaccts++;
											mPStore->save(pData);
											if(cat){
												QString category(cat);
												xml2textEscape(category);
												mPStore->setCategory(pData->mName,category);
												free(cat);
												cat=NULL;
											}
											delete pData;
											pData=NULL;
										}
									}

									//Found a complete set of tags for 
									//account data
									if(strcasecmp(etag,tag)==0){
										if(strcasecmp(tag,"name")==0){
											pData->mName=xml2textEscape(cdata);
											printf("Name: %s\n",pData->mName.latin1());
										}
										if(strcasecmp(tag,"user")==0){
											pData->mUser=cdata;
											pData->mUser=xml2textEscape(cdata);
										}
										if(strcasecmp(tag,"password")==0){
											pData->mPassword=cdata;
											pData->mPassword=xml2textEscape(cdata);
										}
										if(strcasecmp(tag,"note")==0){
											pData->mNote=cdata;
											pData->mNote=xml2textEscape(cdata);
										}
										if(strcasecmp(tag,"category")==0){
											cat = strdup(cdata);
										}
									}
								} else {
										
									//Ok this must be a start-tag!
									// we will save it and compare it
									// to our next end tag
									ungetc(c,in);
									while(!feof(in) && (c=fgetc(in))!='>'){
										if(ti<50){
											tag[ti++]=c;
										}
									}
									tag[ti]='\0';

									//Found the start of an account
									if(strcasecmp(tag,"account")==0){
										if(pData) delete pData;
										pData = new PasswordData();
									}

								}
							}
						} else {
							//Ok we found some data!!!
							ungetc(c,in);
							di=0;
							while(!feof(in) && (c=fgetc(in))!='<'){
								if(di<9000)
									cdata[di++]=c;
							}
							if(c=='<')
								ungetc(c,in);
							cdata[di]='\0';
						}
					}
					fclose(in);
					showAccountList();
					QString msg;

					msg.sprintf("%d Accounts added\n\t%d New\n\t%d Duplicate",newaccts+renaccts+overacct,newaccts,renaccts+overacct);
					QMessageBox::information(this,tr("Added accounts"),
						msg,QMessageBox::Ok);
				} else {
					QMessageBox::critical(this,tr("Error"),tr("Unable to open file"),QMessageBox::Ok,QMessageBox::NoButton);
				}
			}


		
}

