#include <qlineedit.h>
#include <qmultilineedit.h>
#include <qmessagebox.h>
#include <qcombobox.h>
#include <qlabel.h>
#include <qpixmap.h>
#include <qdialog.h>
#include <qtimer.h>
#include <qmenubar.h>
#include <qaction.h>
#include <qpe/config.h>

#include "createcategory.h"
#include "categories.h"
#include "accountview.h"
#include "lineinput.h"


#include "createpassword.h"


/* 
 *  Constructs a AccountView which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
AccountView::AccountView(PasswordStore &store, Config *config, QWidget* parent,  const char* name, bool modal, WFlags fl )
    :  AccountViewUI( parent, name, modal, fl ), mStore(store), mConfig(config)
{
	showMaximized();

	setCaption(tr("Keyring: (new account)"));

	connect(wCategory,SIGNAL(activated(const QString & )),this,SLOT(selectCategory(const QString &)));
	
	mData = NULL;

	QPopupMenu *tools = new QPopupMenu(this);
	wMenuBar->insertItem(tr("Tools"),tools);

	QAction *action = new QAction(tr("Create Password"),tr("Create Password"),0,this);
	action->addTo(tools);
	connect(action,SIGNAL(activated()),this,SLOT(generatePassword()));
	
	updateCategories();

	mTimer = new QTimer(this);
	mTimer->start(500,false);	
	mTotalTime=mConfig->readNumEntry("av.timervalue",60);
	mStartTime=time(NULL);
	mEnableTimer=mConfig->readBoolEntry("av.timerenabled",true);
	if(mEnableTimer){	
		connect(mTimer,SIGNAL(timeout()),this,SLOT(timeout()));
	}

	connect(wName,SIGNAL(textChanged(const QString&)),this,SLOT(textChanged(const QString&)));
	connect(wUser,SIGNAL(textChanged(const QString&)),this,SLOT(textChanged(const QString&)));
	connect(wPassword,SIGNAL(textChanged(const QString&)),this,SLOT(textChanged(const QString&)));
	connect(wNote,SIGNAL(textChanged()),this,SLOT(textChanged()));
}

void AccountView::textChanged(){
	mStartTime=time(NULL);
}

void AccountView::textChanged(const QString &){
	mStartTime=time(NULL);
}

void AccountView::timeout(){
	if((time(NULL)-mStartTime)>mTotalTime){
		reject();
	}
}

void AccountView::generatePassword(){
	CreatePassword pwd(this,mConfig);
	mTimer->stop();
	
	if(pwd.exec()==QDialog::Accepted){
		wPassword->setText(pwd.password());

	}
	mStartTime=time(NULL);
	mTimer->start(500,false);
}

void AccountView::updateCategories(){
	QStringList list = mStore.categoryList();

	wCategory->clear();
	list.remove(tr("Default"));
	list.prepend(tr("Default"));
	list.append(tr("..."));
	
	for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it ) {
		QString pname=mStore.getCategoryPixmap((*it));
		QPixmap *pixmap = getCategoryPixmapByName(pname); 
		if(pixmap){
			wCategory->insertItem(*pixmap,(*it));
		} else 
		wCategory->insertItem((*it));
	}
	
	if(mData){
		QString cat = mStore.getCategory(mData->mName);
		wCategory->setCurrentItem(list.findIndex(cat));
	}
}

void AccountView::selectCategory(const QString &category){
	mTimer->stop();	
	if(!mData){
		if(QMessageBox::Yes == QMessageBox::warning(this,tr("Save Changes?"),
							tr("You must save changes before\n setting the category,\n do you want to save your\n changes?"),
							QMessageBox::Yes,QMessageBox::No)){
			storeAccount();
		} else {
			//Reset the timer
			mStartTime=time(NULL);
			mTimer->start(500,false);
			updateCategories();
			return;
		}
	}
		
	if(mData){		
		if(category == "..."){
				CreateCategory cc(&mStore,this);
				if(cc.exec() == QDialog::Accepted){
					if(cc.category() == ""){
						QMessageBox::warning(this,tr("Name?"),
							tr("Please enter a name for the category"),
							QMessageBox::Ok,QMessageBox::NoButton);
			
						//Reset the timer
						mStartTime=time(NULL);
						mTimer->start(500,false);

						updateCategories();
						return;
					}
					mStore.setCategory(mData->mName,cc.category());
					mStore.setCategoryPixmap(cc.category(),cc.pixmap());
					updateCategories();
				}
		} else {
			mStore.setCategory(mData->mName,category);
		}
		updateCategories();
	}	
	//Reset the timer
	mStartTime=time(NULL);
	mTimer->start(500,false);
}

void AccountView::accept(){
	mTimer->stop();
	//Ok we should have only reached here if the user pressed the "Ok" button.
	//We will ask them if they really wanted to save the account.
	if(mData){
		if(	qstrcmp(wName->text(),mData->mName)==0 &
			qstrcmp(wUser->text(),mData->mUser)==0 &&
			qstrcmp(wPassword->text(), mData->mPassword)==0 &&
			qstrcmp(wNote->text(),mData->mNote)==0)  { 
				QDialog::accept();
				return;		
		}
	} 

	if((wName->text().length() + wUser->text().length()+ wPassword->text().length() + wNote->text().length()) == 0 ){
		QDialog::accept();
		return;
	}
	
	if(QMessageBox::Yes == QMessageBox::warning(this,tr("Save Changes?"),
							tr("Do you want to save your changes?"),QMessageBox::Yes,QMessageBox::No)){
		saveAccount();
	} 
	QDialog::accept();
}

void AccountView::loadAccount(QString name){
	mStartTime=time(NULL);
	mData = mStore.load(name);
	if(mData){
		wName->setText(mData->mName);
		wUser->setText(mData->mUser);
		wPassword->setText(mData->mPassword);
		wNote->setText(mData->mNote);

		setCaption("Keyring: " + mData->mName);
		updateCategories();	
		
	}

}



/*  
 *  Destroys the object and frees any allocated resources
 */
AccountView::~AccountView()
{
	mTimer->stop();
	delete mTimer;
	qWarning("Deleting account data from memory");
	delete mData;
    // no need to delete child widgets, Qt does it all for us
}

/* 
 * public slot
 */
void AccountView::deleteAccount()
{
	if(wName->text().length()==0)
		return;
				
	if(QMessageBox::warning(this,
		tr("Delete Account?"),
		tr("Are you sure that you\nwant to delete this account?"),
		QMessageBox::Yes,QMessageBox::No)==QMessageBox::No)
		return;
	

	if(mData)
		mStore.deleteAccount(mData->mName);
	QDialog::accept();
}

/* 
 * public slot
 */
void AccountView::saveAccount(){
	storeAccount();
	if(mData){ 
			delete mData;
			mData=NULL;
	}
	QDialog::accept();
}

void AccountView::storeAccount()
{
	
	if(wName->text().length()==0) 
			//&& (wUser->text().length()>0 || wPassword->text().length()>0 || wNote->text().length()>0))
			{

			QMessageBox::warning(this,tr("Missing Account Name"),
							tr("Please name this account\nbefore saving it."),
							QMessageBox::Ok,QMessageBox::NoButton);

			return;

	}

		
	if(wName->text().length()>0){
		if(!mData){
			if(mStore.accountExists(wName->text())){
				if(QMessageBox::warning(this,
						tr("Replace Account?"),
						tr("An account with this name\nalready exists, do you want\nto overwrite it?"),
						QMessageBox::Yes,QMessageBox::No)==QMessageBox::No)
						return;

			}
			mData = new PasswordData();
		
			mData->mName = wName->text();
			mData->mPassword = wPassword->text();
			mData->mUser = wUser->text();
			mData->mNote = wNote->text();
		
			
			if(!mStore.save(mData)){
				Error("Failed to save account");
			}
		} else {
			QString oldName = mData->mName;
			
			mData->mName = wName->text();
			mData->mPassword = wPassword->text();
			mData->mUser = wUser->text();
			mData->mNote = wNote->text();
			
			
			if(!mStore.save(mData)){
				Error("Failed to save account");
			}
			

			//Did the account name change? If so delete
			//the old account.
			if(oldName != wName->text()){
				mStore.setCategory(wName->text(),mStore.getCategory(oldName));
				mStore.deleteAccount(oldName);
			}
		}
	}

	//Let see if there is an category for this account
	// If not we will set it to default.
	if(mStore.getCategory(mData->mName)=="Default"){
		mStore.setCategory(mData->mName,"Default");
	}
		
}

