#include <gdbm.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <ctype.h>
#include <time.h>
#include <errno.h>

#include <qbuffer.h>
#include <qdatastream.h>

#include "blowfish.h"
#include "pstore.h"

void printBuffer(char *data,int len){
	for(int i=0;i<len;i++){
		if(i%60 == 0) printf("\n");
		if(isprint(data[i]))
			printf("%c",data[i]);
		else printf(".");
	}
	printf("\n");
}

void readRandom(char *buf,unsigned int len){
	FILE *rnd = fopen(RANDOM_DEV,"r");
	if(!rnd)
		Error("Unable to open random device");
	if(fread(buf,sizeof(char),len,rnd)==0)
		Errno(errno);
	fclose(rnd);
}

char *PasswordData::encode(unsigned int *len,float version){
	
	QBuffer b;
	b.open(IO_WriteOnly);
	QDataStream ds(&b);

	ds<<mUser;
	ds<<mPassword;
	ds<<mNote;

	*len=b.buffer().size();
	
	int rlen = (((*len)/8 + 1) * 8) - *len;
	char buffer[8];
	readRandom(buffer,8);

	
	*len+=rlen;

	while(rlen>0){
		ds<<buffer[rlen--];
	}

	char *data = (char *) malloc(b.buffer().size());	
	memcpy(data,b.buffer().data(),b.buffer().size());
	return data;
}

bool PasswordData::decode(char *data,unsigned int dlen,float version){
		QByteArray ba;

		ba.duplicate(data,dlen);
		QBuffer b(ba);
		
		b.open(IO_ReadOnly);
		
		QDataStream ds(&b);

		ds>>mUser;
		ds>>mPassword;
		ds>>mNote;

		b.close();

		
		return true;
}

/*--------------------------------------------*/

PasswordStore::PasswordStore(QString name):mDBVersion(DB_VERSION),mName(name){
    
}

PasswordStore *PasswordStore::open(QString &file){
	PasswordStore *store = new PasswordStore();
    store->mDB = gdbm_open((char *) file.latin1(),0,GDBM_SYNC|GDBM_READER|GDBM_WRITER,S_IRUSR|S_IWUSR,NULL);
	if(!store->mDB)
		GDBMErrno(gdbm_errno);

	datum key, data;
	key.dptr="_pwd.data";
	key.dsize=9;

	data=gdbm_fetch(store->mDB,key);
	if(data.dptr==NULL){
		Error("Unable to open database invalid metadata");
	} else {
		QByteArray ba;

		ba.duplicate(data.dptr,data.dsize);
		QBuffer b(ba);
		
		b.open(IO_ReadOnly);
		
		QDataStream ds(&b);

		ds>>store->mDBVersion;
		ds>>store->mFlags;
		ds>>store->mName;

		if(store->mDBVersion != DB_VERSION){
			qWarning("Incompatible database version");
		}
	
		free(data.dptr);
		
		b.close();
	}
	return store;
}


PasswordStore *PasswordStore::create(QString &file,QString &password){
	PasswordStore *store = new PasswordStore();
    store->mDB = gdbm_open((char *) file.latin1(),0,GDBM_WRCREAT|GDBM_SYNC|GDBM_READER,S_IRUSR|S_IWUSR,NULL);
	if(!store->mDB)
		GDBMErrno(gdbm_errno);

	store->initPassword(password);


	//Store information about the database in the database
	QBuffer b;
	b.open(IO_WriteOnly);
	QDataStream ds(&b);

	ds<<store->mDBVersion;
	ds<<store->mFlags;
	ds<<store->mName;

	datum key,data;

	key.dptr="_pwd.data";
	key.dsize=9;
	
	data.dptr=b.buffer().data();
	data.dsize=b.buffer().size();

	gdbm_store(store->mDB,key,data,GDBM_REPLACE);
	
	return store;
}

PasswordStore::~PasswordStore(){
	qWarning("Deleting Password Store from memory");
	qWarning("Reorganizing database");
	gdbm_reorganize(mDB);
	gdbm_close(mDB);
}

void PasswordStore::close(){
    gdbm_close(mDB);
}

bool PasswordStore::setPassword(QString &password){
	mBlowfish.Set_Passwd((char *) password.latin1());
	return true;
}

QStringList PasswordStore::entries(){
	datum key,oldkey;

    if(!mDB)
        Error("Error no open database");
    
	key=gdbm_firstkey(mDB);
	QStringList list;

	while(key.dptr) {
			
		QString s;
		if(strncmp("_pwd",key.dptr,4)!=0 &&
			strncmp("_category.",key.dptr,10)!=0 &&
			strncmp("_catpixmap.",key.dptr,11)!=0				
		){
			s.setLatin1(key.dptr,key.dsize);
			list.append(s);	
		}

		oldkey=key;
		key=gdbm_nextkey(mDB,key);
		free(oldkey.dptr);
	}
    return list;
}

void PasswordStore::initPassword(QString &passwd){
	if(!mDB)
		Error("Error no open database");
	
		
	setPassword(passwd);
	
	
	//Get a buffer filled with random bytes
	char buffer[6*8+1];
	readRandom(buffer,6*8+1);
	
	//Place the word verified somewhere within the buffer
	char *str="verified";
	short loc = (((unsigned char) buffer[6*8])%(5*8));
	strcpy(buffer+loc,str);

	//Encrypt the buffer
	mBlowfish.Encrypt(buffer,6*8);

	//Insert into the db
	datum key,data;
	key.dsize=strlen("_pwd.vfr");
	key.dptr="_pwd.vfr";

	data.dsize=6*8;
	data.dptr=buffer;
	
	gdbm_store(mDB,key,data,GDBM_REPLACE);
	
}

bool PasswordStore::accountExists(QString name){
	datum key;
	
	if(!mDB)
		Error("Error no open database");

	key.dsize=name.length();
	key.dptr=(char *) name.latin1();
	

	return gdbm_exists(mDB,key);
}

QString PasswordStore::getCategoryPixmap(QString category){
	datum key,data;

	category = "_catpixmap."+category;
	
	key.dsize=category.length();
	key.dptr=(char *) category.latin1();

	data=gdbm_fetch(mDB,key);
	if(data.dptr){
		QString pix;
		pix.setLatin1(data.dptr,data.dsize);
		free(data.dptr);
		return pix;
	} else {
		return "none";
	}
}

void PasswordStore::setCategoryPixmap(QString category,QString pix){
	datum key,data;

	category = "_catpixmap."+category;
	
	key.dsize=category.length();
	key.dptr=(char *) category.latin1();

	data.dsize=pix.length();
	data.dptr=(char *)pix.latin1();
	
	if(0>gdbm_store(mDB,key,data,GDBM_REPLACE)){
		Error("Unable to save category pixmap data");
	}
}


QString PasswordStore::getCategory(QString acct){
	datum key,data;

	acct = "_category."+acct;
	
	key.dsize=acct.length();
	key.dptr=(char *) acct.latin1();

	data=gdbm_fetch(mDB,key);
	if(data.dptr){
		QString cat;
		cat.setLatin1(data.dptr,data.dsize);
		free(data.dptr);
		return cat;
	} else {
		return "Default";
	}
}

void PasswordStore::setCategory(QString acct,QString category){
	datum key,data;

	acct = "_category."+acct;
	
	key.dsize=acct.length();
	key.dptr=(char *) acct.latin1();

	data.dsize=category.length();
	data.dptr=(char *)category.latin1();
	
	if(0>gdbm_store(mDB,key,data,GDBM_REPLACE)){
		Error("Unable to save category data");
	}
}

void PasswordStore::renameCategory(QString oldname,QString newname){
	datum key,oldkey;

    if(!mDB)
        Error("Error no open database");
    
	key=gdbm_firstkey(mDB);
	while(key.dptr) {
			
		QString s;
		if(strncmp("_category.",key.dptr,10)==0){
			s.setLatin1(key.dptr,key.dsize);
			QString cat = getCategory(s.mid(10));
			if(cat == oldname){
				setCategory(s.mid(10),newname);
			}
		
		}
		
		oldkey=key;
		key=gdbm_nextkey(mDB,key);
		free(oldkey.dptr);
	}
}

QStringList PasswordStore::categoryList(){
	datum key,oldkey;

    if(!mDB)
        Error("Error no open database");
    
	key=gdbm_firstkey(mDB);
	QStringList list;

	while(key.dptr) {
			
		QString s;
		if(strncmp("_category.",key.dptr,10)==0){
		
			s.setLatin1(key.dptr,key.dsize);
			QString category = getCategory(s.mid(10));
			if(list.contains(category)==0){
				list.append(category);
			}
		}
		
		oldkey=key;
		key=gdbm_nextkey(mDB,key);
		free(oldkey.dptr);
	}
	list.sort();
	
    return list;
}


void PasswordStore::deleteAccount(QString name){
	datum key;
	
	if(!mDB)
		Error("Error no open database");

	key.dsize=name.length();
	key.dptr=(char *) name.latin1();
	
	if(0>gdbm_delete(mDB,key))
			Error("Unable to delete account");
	
	name = "_category." + name;
	
	key.dsize=name.length();
	key.dptr=(char *) name.latin1();
	
	gdbm_delete(mDB,key);
}

bool PasswordStore::checkPassword(){
	datum key,data;
	
	key.dsize=strlen("_pwd.vfr");
	key.dptr="_pwd.vfr";

	data = gdbm_fetch(mDB,key);
	if(data.dptr){
		mBlowfish.Decrypt(data.dptr,data.dsize);
		char *str="verified";
		int j=0;
		for(int i=0;i<data.dsize;i++){
			if(data.dptr[i]==str[j])
				j++;
			else j=0;
			if(j==8){
				free(data.dptr);
				return true;
			}
		}
		free(data.dptr);
	} 
	return false;
}

void PasswordStore::lock(){
	mBlowfish.Reset();
	gdbm_sync(mDB);
}

PasswordData *PasswordStore::load(QString name){
	datum key;
	datum data;

	if(name=="_pwd.vfr" || name=="_pwd.data")
		return NULL;
	
	key.dsize=name.length();
	key.dptr=(char *) name.latin1();
	data=gdbm_fetch(mDB,key);
	if(data.dptr){
		PasswordData *pd = new PasswordData();
		pd->mName=name;
		mBlowfish.Decrypt(data.dptr,data.dsize);
		if(!pd->decode(data.dptr,data.dsize,mDBVersion)){
			memset(data.dptr,0,data.dsize);
			free(data.dptr);
			delete pd;
			return NULL;
		}
		free(data.dptr);
		return pd;
	}
	return NULL;
}

bool PasswordStore::changePassword(QString name,QString newpassword){
	datum key;
	datum data;

	if(name=="_pwd.vfr" || name=="_pwd.data")
		return false;

	if(!checkPassword())
		return false;	
	
	key.dsize=name.length();
	key.dptr=(char *) name.latin1();
	data=gdbm_fetch(mDB,key);
	if(data.dptr){
		Blowfish bf;
		mBlowfish.Decrypt(data.dptr,data.dsize);
	
		
		bf.Set_Passwd((char *) newpassword.latin1());
	
		bf.Encrypt(data.dptr,data.dsize);
		

		if(0>gdbm_store(mDB,key,data,GDBM_REPLACE)){
			Error("Unable to save account data");
			return false;
		}
	
		free(data.dptr);
		return true;
	}
	return false;
}
		
bool PasswordStore::save(PasswordData *d){
	datum key;
	datum data;
	
	if(d->mName=="_pwd.vfr")
		Error("Invalid name");


	key.dsize=d->mName.length();
	key.dptr=(char *) d->mName.latin1();

	unsigned int len;
	data.dptr=d->encode(&len,mDBVersion);
	data.dsize=len;

	if(data.dptr){
		mBlowfish.Encrypt(data.dptr,data.dsize);
		if(0>gdbm_store(mDB,key,data,GDBM_REPLACE)){
			Error("Unable to save account data");
			return false;
		}
		return true;
	} 
	return false;
}

